# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class AWSAccount(AWSProperty):
    props = {
        "EmailAddress": (str, False),
        "Id": (str, False),
        "Name": (str, False),
    }


class AssessmentReportsDestination(AWSProperty):
    props = {
        "Destination": (str, False),
        "DestinationType": (str, False),
    }


class Role(AWSProperty):
    props = {
        "RoleArn": (str, False),
        "RoleType": (str, False),
    }


class AWSService(AWSProperty):
    props = {
        "ServiceName": (str, False),
    }


class Scope(AWSProperty):
    props = {
        "AwsAccounts": ([AWSAccount], False),
        "AwsServices": ([AWSService], False),
    }


class Assessment(AWSObject):
    resource_type = "AWS::AuditManager::Assessment"

    props = {
        "AssessmentReportsDestination": (AssessmentReportsDestination, False),
        "AwsAccount": (AWSAccount, False),
        "Description": (str, False),
        "FrameworkId": (str, False),
        "Name": (str, False),
        "Roles": ([Role], False),
        "Scope": (Scope, False),
        "Status": (str, False),
        "Tags": (Tags, False),
    }
