# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.1.0


from . import AWSObject, AWSProperty
from .validators import boolean


class TransformationConfiguration(AWSProperty):
    props = {
        "Actions": ([str], False),
        "ContentTransformation": (dict, False),
    }


class ObjectLambdaConfiguration(AWSProperty):
    props = {
        "AllowedFeatures": ([str], False),
        "CloudWatchMetricsEnabled": (boolean, False),
        "SupportingAccessPoint": (str, True),
        "TransformationConfigurations": ([TransformationConfiguration], True),
    }


class AccessPoint(AWSObject):
    resource_type = "AWS::S3ObjectLambda::AccessPoint"

    props = {
        "Name": (str, True),
        "ObjectLambdaConfiguration": (ObjectLambdaConfiguration, False),
    }


class AccessPointPolicy(AWSObject):
    resource_type = "AWS::S3ObjectLambda::AccessPointPolicy"

    props = {
        "ObjectLambdaAccessPoint": (str, True),
        "PolicyDocument": (dict, True),
    }
