# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class Assignment(AWSObject):
    resource_type = "AWS::SSO::Assignment"

    props = {
        "InstanceArn": (str, True),
        "PermissionSetArn": (str, True),
        "PrincipalId": (str, True),
        "PrincipalType": (str, True),
        "TargetId": (str, True),
        "TargetType": (str, True),
    }


class AccessControlAttributeValueSourceList(AWSProperty):
    props = {
        "AccessControlAttributeValueSourceList": ([str], False),
    }


class AccessControlAttributeValue(AWSProperty):
    props = {
        "Source": (AccessControlAttributeValueSourceList, True),
    }


class AccessControlAttribute(AWSProperty):
    props = {
        "Key": (str, True),
        "Value": (AccessControlAttributeValue, True),
    }


class InstanceAccessControlAttributeConfiguration(AWSObject):
    resource_type = "AWS::SSO::InstanceAccessControlAttributeConfiguration"

    props = {
        "AccessControlAttributes": ([AccessControlAttribute], False),
        "InstanceArn": (str, True),
    }


class PermissionSet(AWSObject):
    resource_type = "AWS::SSO::PermissionSet"

    props = {
        "Description": (str, False),
        "InlinePolicy": (dict, False),
        "InstanceArn": (str, True),
        "ManagedPolicies": ([str], False),
        "Name": (str, True),
        "RelayStateType": (str, False),
        "SessionDuration": (str, False),
        "Tags": (Tags, False),
    }
