# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class Application(AWSObject):
    resource_type = "AWS::ServiceCatalogAppRegistry::Application"

    props = {
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class Attributes(AWSProperty):
    props = {}


class AttributeGroup(AWSObject):
    resource_type = "AWS::ServiceCatalogAppRegistry::AttributeGroup"

    props = {
        "Attributes": (Attributes, True),
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class AttributeGroupAssociation(AWSObject):
    resource_type = "AWS::ServiceCatalogAppRegistry::AttributeGroupAssociation"

    props = {
        "Application": (str, True),
        "AttributeGroup": (str, True),
    }


class ResourceAssociation(AWSObject):
    resource_type = "AWS::ServiceCatalogAppRegistry::ResourceAssociation"

    props = {
        "Application": (str, True),
        "Resource": (str, True),
        "ResourceType": (str, True),
    }
