# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.1.0


from . import AWSObject, AWSProperty


class Target(AWSProperty):
    props = {
        "TargetAddress": (str, False),
        "TargetType": (str, False),
    }


class NotificationRule(AWSObject):
    resource_type = "AWS::CodeStarNotifications::NotificationRule"

    props = {
        "DetailType": (str, True),
        "EventTypeIds": ([str], True),
        "Name": (str, True),
        "Resource": (str, True),
        "Status": (str, False),
        "Tags": (dict, False),
        "Targets": ([Target], True),
    }
