# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class EksInfo(AWSProperty):
    props = {
        "Namespace": (str, True),
    }


class ContainerInfo(AWSProperty):
    props = {
        "EksInfo": (EksInfo, True),
    }


class ContainerProvider(AWSProperty):
    props = {
        "Id": (str, True),
        "Info": (ContainerInfo, True),
        "Type": (str, True),
    }


class VirtualCluster(AWSObject):
    resource_type = "AWS::EMRContainers::VirtualCluster"

    props = {
        "ContainerProvider": (ContainerProvider, True),
        "Name": (str, True),
        "Tags": (Tags, False),
    }
