# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 39.1.0


from . import AWSObject, AWSProperty


class FederationParameters(AWSProperty):
    props = {
        "ApplicationCallBackURL": (str, False),
        "AttributeMap": (dict, False),
        "FederationProviderName": (str, False),
        "FederationURN": (str, False),
        "SamlMetadataDocument": (str, False),
        "SamlMetadataURL": (str, False),
    }


class Environment(AWSObject):
    resource_type = "AWS::FinSpace::Environment"

    props = {
        "Description": (str, False),
        "FederationMode": (str, False),
        "FederationParameters": (FederationParameters, False),
        "KmsKeyId": (str, False),
        "Name": (str, True),
    }
