# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 39.1.0


from . import AWSObject
from .validators import boolean


class ReportDefinition(AWSObject):
    resource_type = "AWS::CUR::ReportDefinition"

    props = {
        "AdditionalArtifacts": ([str], False),
        "AdditionalSchemaElements": ([str], False),
        "BillingViewArn": (str, False),
        "Compression": (str, True),
        "Format": (str, True),
        "RefreshClosedReports": (boolean, True),
        "ReportName": (str, True),
        "ReportVersioning": (str, True),
        "S3Bucket": (str, True),
        "S3Prefix": (str, True),
        "S3Region": (str, True),
        "TimeUnit": (str, True),
    }
