# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 18.6.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class Channel(AWSProperty):
    props = {
        "channelId": (str, False),
        "channelUri": (str, True),
    }


class ProfilingGroup(AWSObject):
    resource_type = "AWS::CodeGuruProfiler::ProfilingGroup"

    props = {
        "AgentPermissions": (dict, False),
        "AnomalyDetectionNotificationConfiguration": ([Channel], False),
        "ComputePlatform": (str, False),
        "ProfilingGroupName": (str, True),
        "Tags": (Tags, False),
    }
