# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from . import AWSObject


class Project(AWSObject):
    resource_type = "AWS::LookoutVision::Project"

    props = {
        "ProjectName": (str, True),
    }
