# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.2.0


from troposphere import Tags

from . import AWSObject


class EventIntegration(AWSObject):
    resource_type = "AWS::AppIntegrations::EventIntegration"

    props = {
        "Description": (str, False),
        "EventBridgeBus": (str, True),
        "EventFilter": (dict, True),
        "Name": (str, True),
        "Tags": (Tags, False),
    }
