# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from . import AWSObject, AWSProperty


class SnsChannelConfig(AWSProperty):
    props = {
        "TopicArn": (str, False),
    }


class NotificationChannelConfig(AWSProperty):
    props = {
        "Sns": (SnsChannelConfig, False),
    }


class NotificationChannel(AWSObject):
    resource_type = "AWS::DevOpsGuru::NotificationChannel"

    props = {
        "Config": (NotificationChannelConfig, True),
    }


class CloudFormationCollectionFilter(AWSProperty):
    props = {
        "StackNames": ([str], False),
    }


class ResourceCollectionFilter(AWSProperty):
    props = {
        "CloudFormation": (CloudFormationCollectionFilter, False),
    }


class ResourceCollection(AWSObject):
    resource_type = "AWS::DevOpsGuru::ResourceCollection"

    props = {
        "ResourceCollectionFilter": (ResourceCollectionFilter, True),
    }
