# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 39.3.0


from troposphere import Tags

from . import AWSObject
from .validators import boolean


class ResourceShare(AWSObject):
    resource_type = "AWS::RAM::ResourceShare"

    props = {
        "AllowExternalPrincipals": (boolean, False),
        "Name": (str, True),
        "PermissionArns": ([str], False),
        "Principals": ([str], False),
        "ResourceArns": ([str], False),
        "Tags": (Tags, False),
    }
