# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 5.3.0


from . import AWSObject, AWSProperty
from .validators import boolean


class S3(AWSProperty):
    props = {
        "Bucket": (str, True),
        "Key": (str, True),
        "ObjectVersion": (str, False),
    }


class Code(AWSProperty):
    props = {
        "S3": (S3, True),
    }


class GitHubRepository(AWSObject):
    resource_type = "AWS::CodeStar::GitHubRepository"

    props = {
        "Code": (Code, False),
        "EnableIssues": (boolean, False),
        "IsPrivate": (boolean, False),
        "RepositoryAccessToken": (str, True),
        "RepositoryDescription": (str, False),
        "RepositoryName": (str, True),
        "RepositoryOwner": (str, True),
    }
