# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.1.0


from . import AWSObject, AWSProperty


class ExperimentTemplateStopCondition(AWSProperty):
    props = {
        "Source": (str, True),
        "Value": (str, False),
    }


class ExperimentTemplate(AWSObject):
    resource_type = "AWS::FIS::ExperimentTemplate"

    props = {
        "Actions": (dict, False),
        "Description": (str, True),
        "RoleArn": (str, True),
        "StopConditions": ([ExperimentTemplateStopCondition], True),
        "Tags": (dict, True),
        "Targets": (dict, True),
    }
