# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 14.1.0


from troposphere import Tags

from . import AWSObject


class Discoverer(AWSObject):
    resource_type = "AWS::EventSchemas::Discoverer"

    props = {
        "Description": (str, False),
        "SourceArn": (str, True),
        "Tags": (Tags, False),
    }


class Registry(AWSObject):
    resource_type = "AWS::EventSchemas::Registry"

    props = {
        "Description": (str, False),
        "RegistryName": (str, False),
        "Tags": (Tags, False),
    }


class RegistryPolicy(AWSObject):
    resource_type = "AWS::EventSchemas::RegistryPolicy"

    props = {
        "Policy": (dict, True),
        "RegistryName": (str, True),
        "RevisionId": (str, False),
    }


class Schema(AWSObject):
    resource_type = "AWS::EventSchemas::Schema"

    props = {
        "Content": (str, True),
        "Description": (str, False),
        "RegistryName": (str, True),
        "SchemaName": (str, False),
        "Tags": (Tags, False),
        "Type": (str, True),
    }
